/**************************************************************************************
*  Library zum ansprechen eines seriellen EEPROMS an einem beliebigen Controllerport  *
***************************************************************************************
*  Revision: 0.3                                                                      *
*  Status: BETA (testing)                                                             *
*  Date: 07.01.2006                                                                   *
*  Time: 14:09                                                                        *
*  Author: Wilhelm Kulzer, Walderstr. 9, D-93192 Wald                                 *
*  Email: techniker[at]quantentunnel.de                                               *
*  Licensetyp: <BEER-WARE-LICENSE>                                                    *
*  Ohter: Diese Library darf privat frei verwendet werden, solange die Herkunft im    *
*         Source mitdokumentiert wird. Fr gewerbl. Nutzung ist mir dies zustzlich   *
*         mitzuteilen. Ich bernehme keinerlei (direkte sowohl indirekte)             *
*         Haftung von Schden die durch diesen Code verursacht wurden bzw. werden!    *
**************************************************************************************/

#include <avr/io.h>
#include "i2c_eep.h"

char i2c_eep_start ( void );
char i2c_eep_stop ( void );
char i2c_eep_write_byte ( unsigned char Data );
char i2c_eep_read_byte ( unsigned char * Data, unsigned char ackn_check );


/* Schreibroutine Byteweise */
extern char i2c_eep_write ( unsigned int Addr, unsigned char * Data )
{
 if ( Addr > I2C_MEMORY ) { return -3; } // Abbruch, Mehr schreiben als reinpasst iss nicht! ;)
 if ( i2c_eep_start() != 0 ) { return -1; } // Abbruch, Busfehler!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_WRITE ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr >> 8 ) ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr & 0xff ) ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( *Data ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_stop() != 0 ) { return -1; } // Abbruch, Busfehler!
 return 0;
}


/* Leseroutine Byteweise*/
extern char i2c_eep_read ( unsigned int Addr, unsigned char * Data )
{
 if ( Addr > I2C_MEMORY ) { return -3; } // Abbruch, Hier gibts nichts mehr zum lesen! ;)
 if ( i2c_eep_start() != 0 ) { return -1; } // Abbruch, Busfehler!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_WRITE ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr >> 8 ) ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr & 0xff ) ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 I2C_SDA_HIGH; I2C_SCL_HIGH; // Re-Start vorbereiten
 if ( i2c_eep_start() != 0 ) { return -1; } // Abbruch, Bus belegt!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_READ ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_read_byte( Data, 0x01 ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_stop() != 0 ) { return -1; } // Abbruch, Busfehler!
 return 0;
}


/* Schreibroutine Pageweise */
extern char i2c_eep_write_page ( unsigned int Addr, unsigned char * Data, unsigned int Bytes )
{
 unsigned int i;
 if ( ( Addr + Bytes - 1 ) > I2C_MEMORY ) { return -3; } // Abbruch, Mehr schreiben als reinpasst iss nicht! ;)
 if ( Bytes > I2C_PAGE ) { return -4; } // max. Page-gre berschritten!
 if ( i2c_eep_start() != 0 ) { return -1; } // Abbruch, Busfehler!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_WRITE ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr >> 8 ) ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr & 0xff ) ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
 for ( i = 0; i < Bytes; i++ )
 {
  if ( i2c_eep_write_byte( *Data ) != 0 ) { return -2; } // Abbruch, Acknowledge falsch
  Data++;
 } 
 if ( i2c_eep_stop() != 0 ) { return -1; } // Abbruch, Busfehler!
 return 0;
} 


/* Leseroutine Pageweise */
extern char i2c_eep_read_page ( unsigned int Addr, unsigned char * Data, unsigned int Bytes )
{
 unsigned int i;
 if ( ( Addr + Bytes - 1 ) > I2C_MEMORY ) { return -3; } // Abbruch, Soweit kann man nicht lesen! ;)
 if ( Bytes > I2C_PAGE ) { return -4; } // max. Page-gre berschritten!
 if ( i2c_eep_start() ) { return -1; } // Abbruch, Busfehler!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_WRITE ) ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr >> 8 ) ) ) { return -2; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( ( Addr & 0xff) ) ) { return -2; } // Abbruch, Acknowledge falsch
 I2C_SDA_HIGH; I2C_SCL_HIGH; // Re-Start vorbereiten
 if ( i2c_eep_start() ) { return -1; } // Abbruch, Bus belegt!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_READ ) ) { return -2; } // Abbruch, Acknowledge falsch
 for ( i = 0; i < Bytes; i++ )
 {
  i2c_eep_read_byte( Data, 0x00 ); // Daten lesen
  I2C_SDA_OUT; I2C_SDA_LOW; I2C_SCL_HIGH; I2C_SCL_LOW; // Acknowledge ans EEPROM
  Data++;
 } 
 if ( i2c_eep_stop() ) { return -1; } // Abbruch, Busfehler!
 return 0;
}


/* LOWLEVEL: Start_Routine */
char i2c_eep_start ( void )
{
 I2C_SDA_IN;
 I2C_SCL_IN;
 if ( !( I2C_SDA_READ && I2C_SCL_READ ) ) { return -1; } // Abbruch => Bus nicht bereit!
 I2C_SDA_OUT;
 I2C_SDA_LOW;
 I2C_SCL_OUT;
 I2C_SCL_LOW;
 return 0;
}


/* LOWLEVEL: Stop_Routine */
char i2c_eep_stop( void )
{
 I2C_SCL_HIGH;
 I2C_SDA_OUT;
 I2C_SDA_HIGH;
 I2C_SCL_IN;
 I2C_SDA_IN;
 if ( !( I2C_SDA_READ && I2C_SCL_READ ) ) { return -1; } // Abbruch => Busfehler!
 return 0;
}


/* LOWLEVEL: Einzelnes Byte schreiben */
char i2c_eep_write_byte ( unsigned char data )
{
 unsigned char mask = 0x80, i;
 for ( i=0; i<8; i++)
 {
  if ( data & mask )
  {
   // SDA-High ausgeben
   I2C_SDA_HIGH;
   I2C_SCL_HIGH;
   I2C_SCL_LOW;
  }
  else
  {
   // SDA-Low ausgeben
   I2C_SDA_LOW;
   I2C_SCL_HIGH;
   I2C_SCL_LOW;
  }
  mask >>= 1;
 }
 I2C_SDA_IN;
 I2C_SCL_HIGH;
 if ( I2C_SDA_READ ) { return -1; } // Abbruch, Acknowledge falsch
 I2C_SCL_LOW;
 I2C_SDA_OUT;
 I2C_SDA_LOW;
 return 0;
}


/* LOWLEVEL: Einzelnes Byte lesen */
char i2c_eep_read_byte ( unsigned char * Data, unsigned char ackn_check )
{
 unsigned char mask = 0x80, i;
 *Data = 0; // Ausgangspunkt definieren
 I2C_SDA_IN;
 for ( i=0; i<8; i++)
 {
  I2C_SCL_HIGH;
  if ( I2C_SDA_READ ) { *Data |= mask; } // SDA = 1 an Bit i => sichern!
  I2C_SCL_LOW;
  mask >>= 1;
 }
 if ( ackn_check )
 {
  I2C_SCL_HIGH;
  if ( !( I2C_SDA_READ ) ) { return -1; } // Abbruch, Acknowledge falsch
  I2C_SCL_LOW;
 } 
 return 0;
}

