/**************************************************************************************
*  Library zum ansprechen eines seriellen EEPROMS an einem beliebigen Controllerport  *
***************************************************************************************
*  Revision: 0.3                                                                      *
*  Status: BETA (testing)                                                             *
*  Date: 07.01.2006                                                                   *
*  Time: 14:09                                                                        *
*  Author: Wilhelm Kulzer, Walderstr. 9, D-93192 Wald                                 *
*  Email: techniker[at]quantentunnel.de                                               *
*  Licensetyp: <BEER-WARE-LICENSE>                                                    *
*  Ohter: Diese Library darf privat frei verwendet werden, solange die Herkunft im    *
*         Source mitdokumentiert wird. Fr gewerbl. Nutzung ist mir dies zustzlich   *
*         mitzuteilen. Ich bernehme keinerlei (direkte sowohl indirekte)             *
*         Haftung von Schden die durch diesen Code verursacht wurden bzw. werden!    *
**************************************************************************************/


#ifndef I2C_Soft
#define I2C_Soft

/* Erluterungen zu den Grundeinstellungen:
   =========================================
   
	I2C_SDA_MASK	Wert, um den benutzten Portpin auszumaskieren
	I2C_SCL_MASK				-"-
	I2C_PORT		Verwendeter Port (Ausgangsregister)
	I2C_PIN				  -"-        (Eingangsregister)
	I2C_DDR				  -"-        (Richtungsregister)
	I2C_DEVICE		Deviceadresse des verwendeten EEPROMs (siehe Datenblatt)
	I2C_WRITE		Kommandobit fr Schreibanweisung
	I2C_READ			-"-         Leseanweisung
	I2C_MEMORY		Hchste, gltige Adresse des EEPROMs
	I2C_PAGE		max. Pagegre (siehe Datenblatt)
*/	
	
/* Grundeinstellungen */
#define I2C_SDA_MASK	0x01 	// PG0 = SDA
#define I2C_SCL_MASK	0x02 	// PG1 = SCL
#define I2C_PORT		PORTG
#define I2C_PIN			PING
#define I2C_DDR			DDRG
#define I2C_DEVICE		0xa0	// 24C256
#define I2C_WRITE		0		// 24C256
#define I2C_READ		1		// 24C256
#define I2C_MEMORY		0x7fff	// 24C256
#define I2C_PAGE		64		// 24C256


/* LOWLEVEL-Makros */
#define I2C_SDA_OUT		I2C_DDR |= I2C_SDA_MASK
#define I2C_SDA_IN		I2C_DDR &= ~I2C_SDA_MASK; I2C_PORT |= I2C_SDA_MASK
#define I2C_SCL_OUT		I2C_DDR |= I2C_SCL_MASK
#define I2C_SCL_IN		I2C_DDR &= ~I2C_SCL_MASK; I2C_PORT |= I2C_SCL_MASK
#define I2C_SCL_LOW		I2C_PORT &= ~I2C_SCL_MASK
#define I2C_SCL_HIGH	I2C_PORT |= I2C_SCL_MASK
#define I2C_SDA_LOW		I2C_PORT &= ~I2C_SDA_MASK
#define I2C_SDA_HIGH	I2C_PORT |= I2C_SDA_MASK
#define I2C_SDA_READ	I2C_PIN & I2C_SDA_MASK
#define I2C_SCL_READ	I2C_PIN & I2C_SCL_MASK

extern char i2c_eep_write ( unsigned int Addr, unsigned char * Data );
extern char i2c_eep_read ( unsigned int Addr, unsigned char * Data );
extern char i2c_eep_write_page ( unsigned int Addr, unsigned char * Data, unsigned int Bytes );
extern char i2c_eep_read_page ( unsigned int Addr, unsigned char * Data, unsigned int Bytes );

#endif
